import Hero, { ISuperElement } from "@ulixee/hero";
import { Api } from "telegram";
import { sleep } from "telegram/Helpers";
import { settings } from "../Databases/JSONDatabase";
import { timeout, TimeoutError, UnknownError } from "../Helpers/utils";
import { NoNewMessagesError, TelegramWorker } from "../Worker/TelegramWorker";
import { CaptchaError } from "./Errors";
export async function doBotTasks(telegramWorker: TelegramWorker,earnMSG: Api.Message,hero: Hero) {
    let task: Api.Message=earnMSG;
    await earnMSG.buttons![1][2].click({sharePhone: false});
    while (true){
        try{
            task=(await telegramWorker.waitForNewMessages(earnMSG.sender!,task.id,settings.botMessages.botTask,10_000))[0];
        }catch{
            try{
                await telegramWorker.waitForNewMessages(earnMSG.sender!,task.id,settings.botMessages.taskOver,10_000);
            } catch{
                // unknown state
                throw new UnknownError("Unknown state of message due doing bot tasks")
            } finally{
                console.log(`${telegramWorker.worker.phoneNumber}: All bot tasks are completed`)
                return
            }
           
        }
        const skipButton=task.buttons![0][1];
        const link=task.buttons![0][0].url;
        if (link!==undefined){
            let botLink='';
            if (/(^\w+:|^)\/\/(?:t|telegram)\.(?:me|dog)\//i.test(link)){
                botLink='@'+link
                .replace(/(^\w+:|^)\/\/(?:t|telegram)\.(?:me|dog)\//i, '')
                .replace(/(?:\?start=)[A-Za-z0-9]+/i, '');
            } else {
                for (let tries=0;tries<3+1;tries++){
                    if (tries===3){
                        console.log(`${telegramWorker.worker.phoneNumber}:Some error while going to link`);
                        return
                    }
                    try{
                        await hero.goto(link, { timeoutMs: 120_000 });
                    } catch (err){
                        continue
                    } finally {
                        break
                    }
                }
                try {
                    await hero.waitForLoad("AllContentLoaded", { timeoutMs: 120_000 });
                } catch (err) {
                    continue
                }
                try{
                    await timeout(
                        (async ()=>{
                            while (await hero.querySelector("button.g-recaptcha.btn.btn-primary") !== null) {
                                let isCaptchaVisible = false;
                                try {
                                    isCaptchaVisible = await hero.querySelector("input[id='recaptcha-token']").$isVisible
                                } catch (err) {
                                    continue
                                }
                                if (isCaptchaVisible === false) {
                                    let captchaButton: ISuperElement;
                                    try {
                                        captchaButton = await hero.querySelector("button.g-recaptcha.btn.btn-primary").$waitForExists();
                                    } catch {
                                        // throw new UnknownError("There is no any captcha button")
                                        // await skipButton.click({sharePhone: false});
                                        continue
                                    }
                                    if (await captchaButton.$isClickable){
                                        try{
                                            await hero.interact({ move: captchaButton }, {
                                                click: {
                                                    element: captchaButton,
                                                    verification: "exactElement",
                                                },
                                            });
                                        } catch{
                                            continue
                                        } finally{
                                            await sleep(5000);
                                        }
                                    }
                                }
                            }
                        })(),
                        180_000
                    )
                } catch (err){
                    if (!(err instanceof TimeoutError)){
                        throw err
                    } else {
                        console.log(`${telegramWorker.worker.phoneNumber}:Skipping bot task due Timeout`);
                        await skipButton.click({sharePhone: false});
                        continue
                    }
                }
                botLink='@'+(await hero.activeTab.url)
                    .replace(/(^\w+:|^)\/\/(?:t|telegram)\.(?:me|dog)\//i, '')
                    .replace(/(?:\?start=)[A-Za-z0-9]+/i, '');
                
            }
            if (await telegramWorker.isBotBanned({username: botLink})){
                await telegramWorker.unblockTargetBot(botLink);
            }
            const startMessage=await telegramWorker.sendMessage(botLink,{message: '/start'});
            try{
                await telegramWorker.waitForNewMessages(botLink,startMessage.id,1,50_000);
            } catch (err){
                await telegramWorker.blockBot(botLink);
                if (err instanceof NoNewMessagesError){
                    await skipButton.click({sharePhone: false});
                    continue
                } else {
                    await telegramWorker.sendMessage(earnMSG.sender!,{message: settings.botButtons.cancel});
                    console.log(`${telegramWorker.worker.phoneNumber}: Error due doing bot task`,err);
                    return 
                }
            } 
            // await sleep(30_000);
            const message=(await telegramWorker.getMessagesByEntity(botLink))[0];
            const forwardedMessage=(await message.forwardTo(settings.telegramLinks.botLink))!;
            try{
                await telegramWorker.waitForNewMessages(settings.telegramLinks.botLink,forwardedMessage[0].id,settings.botMessages.taskComplete);
            } catch (err){
                await telegramWorker.blockBot(botLink);
                if (err instanceof NoNewMessagesError){
                    await skipButton.click({sharePhone: false});
                    continue
                } else {
                    await telegramWorker.sendMessage(earnMSG.sender!,{message: settings.botButtons.cancel});
                    console.log(`${telegramWorker.worker.phoneNumber}: Error due doing bot task`,err);
                    return 
                }
            }
            await telegramWorker.blockBot(botLink);
        } else {
            console.log('Something strange due doing bot tasks\n',link,task.message)
            return
        }
    }
}
