import { settings } from './JSONDatabase'
import { CSettings } from './JSONDatabase';
import fs from "fs";
import { ISettings } from './ISettings';
describe('JSON Settings tests', () => {
    test('Simple check of loading settings.json', async () => {
        await expect(settings.load()).resolves.not.toThrow();
    });

    test('Simple check of loading .test.json', async () => {

        await expect((new CSettings('.test.json')).load()).resolves.not.toThrow();

        const rawSettingsObject: ISettings = JSON.parse(fs.readFileSync('.test.json', 'utf-8'));
        rawSettingsObject.logLevel = "info";
        rawSettingsObject.shuffleAccounts = false;
        rawSettingsObject.bypassMode = false;
        fs.writeFileSync('.test.json', JSON.stringify(rawSettingsObject));

        const tempSettings = new CSettings('.test.json');
        await expect(tempSettings.load()).resolves.not.toThrow();
        expect(tempSettings.logLevel).toEqual("info");
        expect(tempSettings.shuffleAccounts).toEqual(false);
        expect(tempSettings.bypassMode).toEqual(false);
    })
})

beforeAll(() => {
    if (fs.existsSync('.test.json')) {
        fs.rmSync('.test.json');
    }
})

afterAll(() => {
    if (fs.existsSync('.test.json')) {
        fs.rmSync('.test.json');
    }
})