import { difference, isObject } from "lodash";
import { JsonDB, Config } from "node-json-db";
import { IFaucetMail } from "./IFaucetMail";
import { ISettings } from "./ISettings";

export class JSONFileIsBroken extends Error {
    constructor(filename: string) {
        super(`${filename} is broken`);
    }
}

abstract class JSONDatabase {
    protected readonly rawDB: JsonDB;
    isLoaded: boolean = false;
    readonly filename: string
    constructor(filename: string) {
        this.rawDB = new JsonDB(new Config(filename, true, true, '/'));
        this.filename = filename;
    }


    async load() {
        if (this.isLoaded === false) {
            try {
                await this.rawDB.load();
                this.isLoaded = true;
            } catch {
                throw new JSONFileIsBroken(this.filename)
            }
        }
    }
}

export class CJSONFaucetMail extends JSONDatabase {
    constructor(filename: string) {
        super(filename);
    }
    async getWallets(): Promise<IFaucetMail[]> {
        const wallets: IFaucetMail[] = [];
        if (this.isLoaded) {
            const rawWallets = await this.rawDB.getData("/");

            for (const rawWallet of Object.keys(rawWallets)) {
                try {
                    const wallet: IFaucetMail = {
                        faucet: {
                            username: rawWallets[rawWallet].faucetpay.username,
                            password: rawWallets[rawWallet].faucetpay.password,
                        },
                        mail: {
                            address: rawWallet,
                            password: rawWallets[rawWallet].mail_passphrase,
                        },
                    };
                    wallets.push(wallet);
                } catch (e) {
                    continue;
                }
            }

        }
        return wallets
    }
}


export class CSettings extends JSONDatabase implements ISettings {
    logLevel: "debug" | "info" | "error" = "debug";
    pararels: number = 3;
    minimalToWithdraw: number = 0.003;
    maxRefferals: number = 3;
    botMessages = {
        verification: "To continue using this bot,",
        tasksSelector: "Choose an option to start earning your TRX",
        taskOver: "there are no new ads",
        taskComplete: "Success, ",
        notInGroup: "not entered the chat",
        oldMessage: "forwarded is old",
        activeOperation: "to use another command or function",
        blocked: "blocked me at some point",
        welcome: "Here you can earn Tron(TRX) ",
        siteTask: "🔎 Go to website'",
        botTask:"Press the 'Go to bot' button below"
    };
    botButtons = { 
        earn: "❇️ Earn cryptocurrency", 
        balance: "💰 Balance", 
        withdraw: "📤 Withdraw",
        cancel: "❌ Cancel" 
    };
    telegramLinks = { 
        botLink: "@hkearn_trx_bot", 
        groupsToJoin: ["@hkearn_transactions", "@hkearn_updates"],
        botsToSkip: ["@ClickBeeLTCBot","@adbchbot"] 
    };
    sleepTime = { afterDoing: [3600_000, 7200_000], betweenSessions: [35_000, 70_000] };
    shuffleAccounts: boolean = true;
    bypassMode: boolean = true;
    constructor(filename = 'settings.json') {
        super(filename);
    }

    async load() {
        await super.load();
        if (Object.keys((await this.rawDB.getData('/'))).length === 0) {
            const { ...object } = this as Omit<this, "isLoaded" | "filename">
            delete object['rawDB'];
            delete object['isLoaded'];
            delete object['filename'];
            await this.rawDB.push('/', object);
        } else {
            const settings = (await this.rawDB.getData('/'));
            for (const setting in settings) {
                // need check of avalible in object
                if (Object.keys((settings[setting] as Object)).length!==0){
                    if (difference(Object.keys(this[setting]),Object.keys(settings[setting])).length===0){
                        this[setting]=settings[setting];
                    } else {
                        console.log(`Setting ${setting.toUpperCase()} is broken!`)
                    }
                } else {
                    this[setting] = settings[setting];
                }
                
            }
        }
    }
}

export const settings = new CSettings();