import findFreePorts from "find-free-ports";
import { DockerController, wrappedContainer } from "./Docker"
import { getIP } from "../Helpers/utils";
import { differenceWith, isEqual } from "lodash";
import { CSQLDatabaseWorkers } from "../Databases/SQLDatabase";
import { IWorker } from "../Worker/IWorker";
import { WorkerController } from "../Worker/WorkerController";
import fs from "fs";
const AMOUNTTOTESTDOCKERS = 3;
describe('Docker tests', () => {
    
    test("Is Pulled?", async () => {
        await expect(DockerController.isPulled()).resolves.toBeTruthy();
    })

    test('Are containers creating normally?', async () => {
        const ports = await findFreePorts(AMOUNTTOTESTDOCKERS * 3);
        expect(ports).toHaveLength(AMOUNTTOTESTDOCKERS * 3);
        let promises: Promise<wrappedContainer>[] = []
        for (let i = 0; i != AMOUNTTOTESTDOCKERS; i++) {
            promises.push(DockerController.createNewDocker({
                http: ports[i * 3],
                socks: ports[i * 3 + 1],
                control: ports[i * 3 + 2]
            }));
        }
        await expect(Promise.all(promises)).resolves.toHaveLength(AMOUNTTOTESTDOCKERS);
    }, 400_000)
    test("Are containers working normally?", async () => {
        expect(DockerController.containers).toHaveLength(AMOUNTTOTESTDOCKERS);
        const ipRegExp = /^((25[0-5]|(2[0-4]|1\d|[1-9]|)\d)(\.(?!$)|$)){4}$/
        const localIP = await getIP();
        expect(localIP).toMatch(ipRegExp);
        for (const container of DockerController.containers) {
            let testIP: string;
            testIP = await container.getIP()
            expect(testIP).toMatch(ipRegExp);
            expect(testIP).not.toEqual(localIP);
        }

    }, 10_000)
    test("Are changing ip work normaly?", async () => {
        expect(DockerController.containers).toHaveLength(AMOUNTTOTESTDOCKERS);
        for (const container of DockerController.containers) {
            const prev_ip = await container.getIP();
            await expect(container.changeIP()).resolves.not.toThrow();
            await expect(container.getIP()).resolves.not.toEqual(prev_ip);
        }
        // second time
        for (const container of DockerController.containers) {
            const prev_ip = await container.getIP();
            await expect(container.changeIP()).resolves.not.toThrow();
            await expect(container.getIP()).resolves.not.toEqual(prev_ip);
        }

    }, 200_000)

})

describe('Worker Controller test', () => {

    const sampleDB: CSQLDatabaseWorkers = new CSQLDatabaseWorkers('.test.sqlite');

    test('Adding a worker to test database', async () => {
        await expect(sampleDB.load()).resolves.not.toThrow();
        expect(DockerController.containers).toHaveLength(AMOUNTTOTESTDOCKERS);
        for (let i = 0; i != AMOUNTTOTESTDOCKERS; i++) {
            const sampleWorker: Omit<IWorker, "browserFingerprint" | "canBeRefferal"> = {
                faucetMail: { faucet: { username: (Math.random() + 1).toString(36).substring(7), password: "" },mail:{address:''} },
                phoneNumber: (Math.random() + 1).toString(36).substring(7),
                telegramID: Number((Math.random() * 10000000).toFixed(0)),
                apiID: 0,
                apiHash: "",
                password: "",
                stringSession: (Math.random() + 1).toString(36).substring(7),
                balance: 0,
                refferal: null,
                troubled: false,
                isMailed: true
            }
            await expect(sampleDB.addWorker(sampleWorker)).resolves.not.toThrow();
        }
    })
    test('Is iterating normally?', async () => {
        expect(DockerController.containers).toHaveLength(AMOUNTTOTESTDOCKERS);
        await expect(sampleDB.getWorkers()).resolves.toHaveLength(AMOUNTTOTESTDOCKERS);
        const iterator = new WorkerController()
        await expect(iterator.load(sampleDB)).resolves.not.toThrow();
        const testInterate = async (i: number) => {
            const workers: IWorker[] = [];
            for await (const worker of iterator.interateWorkers(DockerController.containers[i])) {
                if (worker !== undefined) {
                    workers.push(worker);
                } else {
                    break
                }
            }
            return workers;
        }
        const promises: Promise<IWorker[]>[] = []
        for (let i = 0; i != AMOUNTTOTESTDOCKERS; i++) {
            promises.push(
                testInterate(i)
            )
        }
        await expect(Promise.all(promises)).resolves.toHaveLength(AMOUNTTOTESTDOCKERS);
        const results = await Promise.all(promises);
        for (let i = 0; i != AMOUNTTOTESTDOCKERS; i++) {
            if (i + 1 < AMOUNTTOTESTDOCKERS) {
                expect(differenceWith(results[i], results[i + 1], isEqual)).toHaveLength(1);
            }
        }
    })
    afterAll(async () => {
        await sampleDB.disconnect();
        if (fs.existsSync('.test.sqlite')) {
            fs.rmSync('.test.sqlite');
        }
    })
})

afterAll(async () => {
    await DockerController.killContainers();
})