import prompts from "prompts";
import { db } from "../Databases/SQLDatabase";
import fs from "fs";
import { CJSONFaucetMail, settings } from "../Databases/JSONDatabase";
import { TelegramWorker } from "../Worker/TelegramWorker";
import { StringSession } from "telegram/sessions";
import { LogLevel } from "telegram/extensions/Logger";
import * as telegram from "telegram";
import { IFaucetMail } from "../Databases/IFaucetMail";

export async function addWorkers() {
    const workers = await db.getWorkers();

    const usePrevApi = workers.length > 0 ? (await prompts({
        type: "confirm",
        name: "answer",
        message: "Use apiID and apiHash from last added worker?"
    })).answer : false
    if (usePrevApi === undefined) {
        return
    }
    const [apiID, apiHash] = usePrevApi ? [workers[workers.length - 1].apiID, workers[workers.length - 1].apiHash] : Object.values(await prompts([
        {
            type: "number", name: "apiID", message: "Enter valid apiID:", validate: (apiID: number) => {
                if (!apiID) {
                    return "Enter apiID!"
                }
                return true
            }
        },
        {
            type: "text", name: "apiHash", message: "Enter valid apiHash:", format: apiHash => apiHash.trim().replace(/\ /gm, ""), validate: (apiHash: string) => {
                apiHash=apiHash.trim().replace(/\ /gm, "");
                if (apiHash.length === 0) {
                    return "Enter apiHash!"
                }
                return true
            }
        }
    ])) as [number, string]
    if (apiID === undefined || apiHash === undefined) {
        return
    }
    
    while (true) {
        const { phoneNumber, password } = await prompts([
            {
                type: "text",
                name: "phoneNumber",
                message: "Phone number?",
                format: phoneNumber => phoneNumber.trim().replace(/\ /gm, ""),
                validate: (phoneNumber: string) => {
                    phoneNumber=phoneNumber.trim().replace(/\ /gm, "");
                    if (/^[\+]?[(]?[0-9]{3}[)]?[-\s\.]?[0-9]{3}[-\s\.]?[0-9]{4,6}$/gmi.test(phoneNumber) === false) {
                        return "Not valid phone number!"
                    } else if (workers.filter(worker => worker.phoneNumber === phoneNumber).length !== 0) {
                        return "Account with that phone number already exists!" // need better log
                    }
                    return true
                }
            },
            {
                type: "text",
                name: "password",
                message: "2FA Password?"
            },
        ]);
        if (phoneNumber === undefined || password === undefined) {
            return
        }
        let mail: IFaucetMail | false=false;
        for (const worker of await db.getWorkers()){
            const sameMail=(await db.getWorkers()).filter((_worker)=>{
                return _worker.phoneNumber!==worker.phoneNumber && _worker.faucetMail.mail.address===worker.faucetMail.mail.address
            });
            if (sameMail.length===0){
                mail=worker.faucetMail
            }
        }
        const isMailed= mail!==false ? !((await prompts({
            type:"confirm",
            message:"Want to use TRX wallet of mail?",
            name:'isMailed'
        })).isMailed) : true // false=trx wallet true=mail
        
        
        if (isMailed===true){
            const jsonFiles: prompts.Choice[] = [];
            for (const file of fs.readdirSync("./")) {
                if (file.includes(".json") && file != settings.filename) {
                    const jsonFile = new CJSONFaucetMail(file);
                    try {
                        await jsonFile.load();
                    } catch (e) {
                        continue;
                    }
                    const wallets = await jsonFile.getWallets();
                    if (wallets.length > 0 && (await db.getWallet(wallets)) !== undefined) {
                        jsonFiles.push({
                            title: file,
                            description: "json file",
                            value: jsonFile,
                        });
                    }

                }
            }
            if (jsonFiles.length === 0) {
                throw new Error("There is no valid `.json` file");
            }
            const walletsFile: CJSONFaucetMail = (
                await prompts({
                    type: "select",
                    name: "file",
                    message: "Choose wallets json file",
                    choices: jsonFiles,
                })
            ).file;
            if (walletsFile === undefined) {
                return
            }
            mail=(await db.getWallet(await walletsFile.getWallets()))!
        } else {
            if (mail===false){
                throw new Error("There is no available mails for second use");
            }
        }

        const telegramWorker = new TelegramWorker({
            phoneNumber:phoneNumber,
            apiID:apiID,
            apiHash:apiHash,
            stringSession:""
        });
        telegramWorker.setLogLevel(LogLevel.ERROR);
        try {
            await telegramWorker.start({
                phoneNumber: phoneNumber,
                password: async () => password,
                phoneCode: async () =>
                    (
                        await prompts({
                            type: "number",
                            name: "code",
                            message: "Enter recieved code",
                        })
                    ).code.toString(),
                onError: (err) => {
                    console.log(`Error due singning into telegram worker\n${err}`); // need better log
                    throw err;
                },
            });
        } catch (err) {
            console.log(err)
            continue;
        }

        if ((await telegramWorker.isUserAuthorized()) === true) {
            const me = (await telegramWorker.getMe(false)) as telegram.Api.User;
            await db.addWorker({
                phoneNumber: phoneNumber,
                password: password,
                faucetMail: mail,
                telegramID: me.id.toJSNumber(),
                apiID: apiID,
                apiHash: apiHash,
                stringSession: String(telegramWorker.session.save()),
                balance: 0,
                refferal: await db.getRefferal(),
                isMailed: isMailed
            })

        } else {
            console.log('TelegramWorker is not authorized'); // need better log
        }
        if ((await prompts({ type: "confirm", name: "continue", message: "Continue?", })).continue != true) {
            return;
        }
    }
}

