import { Fingerprint } from "fingerprint-generator";
import { IFaucetMail } from "../Databases/IFaucetMail";
import { db } from "../Databases/SQLDatabase";
import fs from "fs";
import prompts from "prompts";
import { JsonDB, Config } from "node-json-db";

export interface OldAccountCompletedGroupsTasksInterface {
    timeToLeave: number;
    groupID: string;
}
interface OldAccountWithdrawInterface {}
interface OldAccountInterface {
    phoneNumber: string;
    telegramID: number;
    apiID: number;
    apiHash: string;
    password: string;
    faucetMail: Required<IFaucetMail>;
    stringSession: string;
    balance: number;
    withdraws: OldAccountWithdrawInterface[];
    completedGroupsTasks: OldAccountCompletedGroupsTasksInterface[];
    canBeRefferal: boolean;
    browserFingerprint: Fingerprint;
    refferal: number | null;
}
(async ()=>{
    await db.load();
    const jsonFiles: prompts.Choice[] = [];
    fs.readdirSync("./").forEach((file) => {
        if (file.includes(".json")){
            jsonFiles.push({
                title: file,
                description: "json file",
                value: file,
            });
        }
    })
    const path: string=(
        await prompts({
            type: "select",
            name: "file",
            message: "Choose old database json file",
            choices: jsonFiles,
        })
    ).file;
    const oldDatabase = new JsonDB(new Config(path, false, true, "/"));
    
    for (const oldWorker of (await oldDatabase.getObject<OldAccountInterface[]>('/accounts')) ){
        console.log("adding ",oldWorker.phoneNumber);
        await db.addWorker({
            phoneNumber:oldWorker.phoneNumber.trim().replace(/\ /,''),
            password: oldWorker.password,
            apiHash: oldWorker.apiHash,
            apiID: oldWorker.apiID,
            faucetMail:oldWorker.faucetMail,
            telegramID: oldWorker.telegramID,
            stringSession: oldWorker.stringSession,
            balance: oldWorker.balance,
            refferal: oldWorker.refferal,
            canBeRefferal: oldWorker.canBeRefferal,
            isMailed:true
        });
    }
    process.exit();
})();