import Hero from "@ulixee/hero";
import Miner from "@ulixee/miner";
import { shuffle } from "lodash";
import { Api } from "telegram";
import { FloodWaitError } from "telegram/errors";
import { sleep } from "telegram/Helpers";
import { doBotTasks } from "../Automatization/Bots";
import { doSiteTasks } from "../Automatization/Sites";
import { verification } from "../Automatization/Verification";
import { settings } from "../Databases/JSONDatabase";
import { db } from "../Databases/SQLDatabase";
import { wrappedContainer } from "../DockerWrapper/Docker";
import { NoNewMessagesError, TelegramWorker } from "../Worker/TelegramWorker";
import { WorkerController } from "../Worker/WorkerController";
import { initHero, timeout, TimeoutError} from "./utils";


export async function processWorkers(dockerContainer: wrappedContainer, workerController: WorkerController, heroMiner: Miner) {
    mainLoop: while (true) {
        const iteration = (await workerController.interateWorkers(dockerContainer).next());
        if (iteration.done !== true) {
            const worker = iteration.value;
            console.log(`Current worker:${worker.phoneNumber}\nBalance:${worker.balance}`);
            while (workerController.isFreeIP(worker, await dockerContainer.getCurrentIP()) === false) {
                try {
                    await dockerContainer.changeIP();
                } catch (err) {
                    // need to do more stable
                    console.log('Some error due changing ip', err);
                    workerController.push_back(worker);
                    continue mainLoop
                }

            }
            async function prepareToContinueMainLoop() {
                workerController.removeLastIP(worker);
                workerController.push_back(worker);
            }
            const telegramWorker = new TelegramWorker(worker);
            await telegramWorker.connect();

            if ((await telegramWorker.checkAuthorization()) === false) {
                console.log(
                    `Account ${worker.phoneNumber} is not authorizated`
                );
                worker.troubled = true;
                await db.updateWorker(worker);
                continue;
            } else {
                console.log(`${dockerContainer.name}:Successfully connected to account ${worker.phoneNumber}`);
                let botEntity: Api.User;
                try {
                    botEntity = await telegramWorker.checkFloodErrorByBotEntity();
                    await telegramWorker.leaveOldGroups();
                    await telegramWorker.isJoinedToSettingsGroup();
                    await telegramWorker.unblockMainBot();
                }
                catch (err) {
                    workerController.removeLastIP(worker);
                    if (err instanceof FloodWaitError) {
                        console.log(`Flood Wait error for ${err.seconds / 3600} hours\n`, worker.phoneNumber);
                    } else {
                        console.log('Error while doing main telegram activity\n', worker.phoneNumber, err);
                        workerController.push_back(worker);
                    }
                    continue mainLoop;
                }
                // await sleep(Math.random() * 50_000);
                if ((await telegramWorker.getMessagesByEntity(botEntity)).length == 0) { // first time activation

                    if (worker.refferal !== null) {
                        await telegramWorker.sendMessage(botEntity, {
                            message: `/start ${worker.refferal}`,
                        })
                    } else {
                        await telegramWorker.sendMessage(botEntity, {
                            message: `/start`,
                        })
                    }

                }
                await sleep(Math.random() * 10_000);
                const hero = await initHero(worker, { ip: await dockerContainer.getCurrentIP(), socks: dockerContainer.ports.socks }, heroMiner) as Hero;
                tasksSelectorLoop: while (true) {
                    let earn = await telegramWorker.sendMessage(botEntity, { message: settings.botButtons.earn });
                    
                    let tasksSelector: Api.Message;
                    try {
                        tasksSelector=(await telegramWorker.waitForNewMessages(botEntity, earn.id,
                            [
                                settings.botMessages.activeOperation,
                                settings.botMessages.verification,
                                settings.botMessages.tasksSelector
                            ],
                            60_000
                            ))[0];
                    } catch (err) {
                        await prepareToContinueMainLoop();
                        await hero.close();
                        if (err instanceof NoNewMessagesError) {
                            let unknownMessage: Api.Message;
                            try{
                                unknownMessage=await telegramWorker.waitForNewMessages(botEntity, earn.id)[0];
                            } catch (err){
                                if (err instanceof NoNewMessagesError) {
                                    console.log('Is bot working?');
                                    continue mainLoop
                                } else  {
                                    throw err
                                }
                            }
                            await telegramWorker.sendMessage(botEntity, { message: settings.botButtons.cancel });
                            await sleep(10_000);
                            await telegramWorker.sendMessage(botEntity, { message: '/start' });
                            console.log(`Worker ${worker.phoneNumber}:Unknown message:`, unknownMessage.message);
                            await prepareToContinueMainLoop();
                            await hero.close();
                            continue mainLoop
                        } else {
                            throw err
                        }
                    }
                    
                    if (tasksSelector.message.includes(settings.botMessages.activeOperation)){
                        const cancel_message = await telegramWorker.sendMessage(botEntity, { message: settings.botButtons.cancel });
                        try {
                            await telegramWorker.waitForNewMessages(botEntity, cancel_message.id, 2);
                        } catch (err) {
                            await prepareToContinueMainLoop();
                            await hero.close();
                            if (err instanceof NoNewMessagesError) {
                                console.log('Is bot working?');
                                continue mainLoop
                            } else {
                                throw err
                            }
                        }
                    } else if (tasksSelector.message.includes(settings.botMessages.verification)) {
                        console.log(`Worker ${worker.phoneNumber} is not verificated!`);
                        try{
                            await timeout(verification(tasksSelector.buttons![0][0].url!, hero),300_000);
                        } catch (err){
                            if (err instanceof TimeoutError){
                                console.log('Vertification is reached timeout');
                                await prepareToContinueMainLoop();
                                continue mainLoop;
                            } else {
                                throw err
                            }
                        }
                    } else if (tasksSelector.message.includes(settings.botMessages.tasksSelector)) {
                        // taskSelector
                        console.log(`Worker ${worker.phoneNumber}: got task selector`);
                        for (const func of shuffle([doSiteTasks,doBotTasks])){
                            try{
                                await func(telegramWorker,tasksSelector,hero);
                            } catch (err){
                                if (err instanceof NoNewMessagesError){
                                    console.log('Is bot working?');
                                } else if (!(err instanceof TimeoutError)){
                                    await prepareToContinueMainLoop();
                                    await hero.close();
                                    continue mainLoop
                                } else {
                                    console.log(`${func.name} is reached timeout`);
                                }   
                            }
                        }
                        break tasksSelectorLoop
                    } 
                }
                await hero.close();
            }
        } else {
            return
        }
    }

}