import Miner from "@ulixee/miner";
import findFreePorts from "find-free-ports";
import process from "node:process";
import { sleep } from "telegram/Helpers";
import { settings } from "../Databases/JSONDatabase";
import { db } from "../Databases/SQLDatabase";
import { wrappedContainer, DockerController, DOCKERIMAGE } from "../DockerWrapper/Docker";
import { DockerContainerIsBroken } from "../DockerWrapper/DockerErrors";
import { WorkerController } from "../Worker/WorkerController";
import { processWorkers } from "./proccessWorkers";
import { getIP, randomInRange, UnknownError } from "./utils";
export async function start() {
    const amountWorkers = (await db.getWorkers()).length
    if (amountWorkers < settings.pararels) {
        console.log('Pararels too much');
        settings.pararels = amountWorkers
    }
    const ports = await findFreePorts(settings.pararels * 3);
    const promises: Promise<wrappedContainer>[] = [];
    if (await DockerController.isPulled()===false){
        console.log(`Run "docker pull ${DOCKERIMAGE}"`)
        console.log('Restart script...');
        process.exit();
    }
    for (let i = 0; i != settings.pararels; i++) {
        promises.push(DockerController.createNewDocker({
            http: ports[i * 3],
            socks: ports[i * 3 + 1],
            control: ports[i * 3 + 2]
        }));
    }
    try{
        await Promise.all(promises);
    } catch (err){
        await DockerController.killContainers();
        throw err
    }
    const dockers = await Promise.all(promises);;


    console.log("Getting local ip...");
    try {
        process.env["LOCAL_IP"] = await getIP();
    } catch {
        console.log('Error due getting local ip...');
        process.env["LOCAL_IP"] = '';
    }

    console.log("Starting farming...");
    if (dockers.length != settings.pararels) {
        throw new UnknownError('There are not enough docker containers to run properly');
    } else {
        const workerController = new WorkerController();
        while (true) {
            const heroMiner = new Miner();
            await heroMiner.listen();

            await workerController.load();
            const workersPararels: Promise<void>[] = [];
            dockers.forEach((docker) => {
                console.log(docker.ports)
                workersPararels.push(processWorkers(docker, workerController, heroMiner));
            })
            for (const promise of await Promise.allSettled(workersPararels)) {
                if (promise.status === "rejected") {
                    if (promise.reason instanceof Error) {
                        throw promise.reason
                    } else {
                        throw new UnknownError("Some sort of error happened:" + promise.reason as string)
                    }

                }
            }
            console.log('Closing miner...');
            await heroMiner.close(true);
            // await heroMiner.router.close(); // close all hero's
            console.log("Doned")
            // await DockerController.killContainers();
            if (workerController.availableWorkers.length === 0) {
                await sleep(randomInRange(settings.sleepTime.betweenSessions[0], settings.sleepTime.betweenSessions[1]));
            } else {
                console.log('oh no, there is a few workers available');
            }

        }


    }

}