import { JSONFileIsBroken, settings } from "./Databases/JSONDatabase"
import { db } from "./Databases/SQLDatabase";
import prompts from "prompts";
import { addWorkers } from "./Helpers/addWorkers";
import { start } from "./Helpers/startFarm";
import { DockerController } from "./DockerWrapper/Docker";



(async () => {
    try {
        await settings.load();
        await db.load();
    } catch (err) {
        if (err instanceof JSONFileIsBroken) {
            // log what settings.json is broken
        } else {
            // log what something went wrong...
        }
        process.exit();
    }
    while (true) {
        const workers = await db.getWorkers();

        const answer = await prompts({
            type: "select",
            name: "menu",
            message: "Choose action",
            choices: [
                workers.length > 0
                    ? {
                        title: "Start",
                        description: `Starting farm for ${workers.length} accounts`,
                        value: "start",
                    }
                    : {
                        title: "Start",
                        description: "Add accounts first...",
                        value: "start",
                        disabled: true,
                    },
                {
                    title: "Add accounts",
                    description: "Add accounts to database",
                    value: "addAccounts",
                },
                workers.filter((worker) => { return worker.troubled }).length > 0
                    ? {
                        title: "Solve issue with max amount of groups",
                        description: `Solve issue for ${workers.filter((worker) => { return worker.troubled }).length} accounts`,
                        value: "leaveGroups",
                    }
                    : {
                        title: "Solve issue with max amount of groups",
                        description: `Nothing to solve`,
                        value: "leaveGroups",
                        disabled: true,
                    },
                {
                    title: "Check balances",
                    value: "checkBalances",
                    disabled: true,
                },
                { title: "Withdraw", value: "withdraw", disabled: true },
                {
                    title: "Exit",
                    description: "You realy dont know what it mean?",
                },
            ],
        });
        switch (answer.menu) {
            case "addAccounts":
                await addWorkers();
                break;
            case "start":
                await start();
                break;
            default:
                // logger.info("Bye bye");
                await DockerController.killContainers();
                process.exit();
        }
    }
})()
